/*
    DB Port
    Copyright (C) 2010 Daniel Mewes
*/

package dbport;

import java.util.*;

import systeminterface.*;
import metadata.*;

class CreateTablePage extends HtmlPage {
    public static String getContent(DbEnvironment environment, String[] arguments) throws Exception {
        StringBuffer result = new StringBuffer();
        
        result.append(getHtmlHead());
        
        if (arguments.length > 0) {
            // Try to create table
            
            String name = null;
            String columns = null;
            
            for (String arg : arguments) {
                if (arg.length() >= 5 && arg.substring(0, 5).equals("name="))
                    name = arg.substring(5);
                else if (arg.length() >= 8 && arg.substring(0, 8).equals("columns="))
                    columns = arg.substring(8);
                else {
                    result.append("<h1>Invalid argument</h1>");
                    break;
                }
            }
            
            if (name != null && columns != null) {                                
                result.append("<h1>Table created</h1>");
                result.append("Tablename: " + name + "<br><br>");
                
                Map<String, Type> schema = new HashMap<String, Type>();
                
                String[] columnLines = columns.split("\n");
                for (String line : columnLines) {
                    String[] lineNameType = line.split(":");
                    if (lineNameType.length == 2) {
                        String columnName = lineNameType[0].trim();
                        String columnTypeName = lineNameType[1].trim();
                        Type columnType = null;
                        if (columnTypeName.equals("Integer"))
                            columnType = Types.getIntegerType();
                        else if (columnTypeName.equals("Float"))
                            columnType = Types.getFloatType();
                        else if (columnTypeName.equals("Double"))
                            columnType = Types.getDoubleType();
                        else if (columnTypeName.equals("Date"))
                            columnType = Types.getDateType();
                        else if (columnTypeName.equals("Long"))
                            columnType = Types.getLongType();
                        else if (columnTypeName.equals("Varchar"))
                            columnType = Types.getVarcharType();
                        else if (columnTypeName.substring(0, 5).equals("Char(")) {
                            if (columnTypeName.indexOf(")") >= 0) {
                                String length = columnTypeName.substring(5, columnTypeName.indexOf(")"));
                                columnType = Types.getCharType(Integer.decode(length).intValue());
                            }
                        }
                        
                        if (columnType == null)
                            result.append("Invalid column<br>");
                        else {
                            schema.put(columnName, columnType);
                            result.append("Column: " + columnName + " : " + columnType.getName() + "<br>");
                        }
                    }
                }
                
                environment.database.getQueryInterface().createTable(name, schema);
            }
            else
                result.append("<h1>Missing argument</h1>");
        }
        else {
            result.append("<h1>Create table</h1>");
            
            result.append("<form method=\"get\" action=\"#\">\n");
            result.append("<table border=\"0\">\n");
            
            result.append("<tr><td>Name: </td>");
            result.append("<td><input name=\"name\"></td>");
            result.append("</tr>");
            
            result.append("<tr><td>Columns: </td>");
            result.append("<td><textarea rows=\"10\" cols=\"40\" name=\"columns\">columnName1 : Integer\ncolumnName2 : Char(5)\n...</textarea></td>");
            result.append("</tr>");
            
            result.append("</table>");
            result.append("<input type=\"submit\">");
            result.append("</form>");
        }
        
        result.append(getHtmlTail());
        
        return result.toString();
    }
}
