/*
    DB Port
    Copyright (C) 2010 Daniel Mewes
*/

package dbport;

import java.util.*;
import java.text.*;

import systeminterface.*;
import metadata.*;
import operator.*;

class InsertDataPage extends HtmlPage {
    public static String getContent(DbEnvironment environment, String[] arguments) throws Exception {
        StringBuffer result = new StringBuffer();
        
        result.append(getHtmlHead());
        
        if (arguments.length > 0) {
            // Try to insert row(s)
            
            String repeat = null;
            String table = null;
            String columns = null;
            String data = null;
            
            for (String arg : arguments) {
                if (arg.length() >= 7 && arg.substring(0, 7).equals("repeat="))
                    repeat = arg.substring(7);
                else if (arg.length() >= 8 && arg.substring(0, 8).equals("columns="))
                    columns = arg.substring(8);
                else if (arg.length() >= 5 && arg.substring(0, 5).equals("data="))
                    data = arg.substring(5);
                else if (arg.length() >= 6 && arg.substring(0, 6).equals("table="))
                    table = arg.substring(6);
                else {
                    result.append("<h1>Invalid argument</h1>");
                    break;
                }
            }
            
            if (table != null && columns != null && data != null) {   
                int repetitions = 1;
                if (repeat != null) {
                    repetitions = Integer.decode(repeat).intValue();
                }
                                         
                result.append("<h1>" + repetitions + " Row(s) inserted</h1>");
                result.append("Table name: " + table + "<br><br>");
                
                Map<String, Type> schema = new HashMap<String, Type>();
                
                Operator<? extends Column> columnOperator = environment.database.getStorageInterface().getTableByName(table).getAllColumns();
                columnOperator.open();
                Column currentColumn;
                while ((currentColumn = columnOperator.next()) != null)
                    schema.put(currentColumn.getColumnName(), currentColumn.getColumnType());
                columnOperator.close();
                
                String[] columnNames = columns.split(",");
                
                String[] values = data.split(",");
                
                Random rnd = new Random();
                
                
                long startTime = System.currentTimeMillis();
                
                while (repetitions > 0) {
                    PrimitiveRow row = new PrimitiveRow(schema);
                    
                    for (int i = 0; i < columnNames.length; ++i) {
                        String columnName = columnNames[i].trim();
                        String value = values[i].trim();
                        Object valueObject = null;
                        
                        if (value.equals("rnd()")) {
                            Type valueType = schema.get(columnName);
                            if (valueType.getName().equals("Integer"))
                                valueObject = new Integer(rnd.nextInt());
                            else if (valueType.getName().equals("Long"))
                                valueObject = new Long(rnd.nextLong());
                            else if (valueType.getName().equals("Float"))
                                valueObject = new Float(rnd.nextFloat());
                            else if (valueType.getName().equals("Double"))
                                valueObject = new Double(rnd.nextDouble());
                            else if (valueType.getName().equals("Date"))
                                valueObject = new Date(Calendar.getInstance().getTime().getTime() + 1000000l * (long)rnd.nextInt(1000000) - 500000000000l);
                            else // Hope we have a String type...
                                valueObject = null; // Random strings not supported currently :-(
                        }
                        else if (value.equals("null"))
                            valueObject = null;
                        else {
                            Type valueType = schema.get(columnName);
                            if (valueType.getName().equals("Integer"))
                                valueObject = new Integer(value);
                            else if (valueType.getName().equals("Long"))
                                valueObject = new Long(value);
                            else if (valueType.getName().equals("Float"))
                                valueObject = new Float(value);
                            else if (valueType.getName().equals("Double"))
                                valueObject = new Double(value);
                            else if (valueType.getName().equals("Date"))
                                valueObject = (new SimpleDateFormat("yyyy-MM-DD")).parse(value);
                            else // Hope we have a String type...
                                valueObject = value;
                        }
                            
                        row.setColumnValue(columnName, valueObject);
                    }
                    
                    environment.database.getQueryInterface().insertRow(table, row);
                
                    repetitions--;
                }
                
                result.append("Took " + (System.currentTimeMillis() - startTime) + " ms");
                
            }
            else
                result.append("<h1>Missing argument</h1>");
        }
        else {
            result.append("<h1>Insert row</h1>");
            
            result.append("<form method=\"get\" action=\"#\">\n");
            
            result.append("Run query <input name=\"repeat\" size=\"8\" value=\"1\"> times (Hint: You may want to use rnd() as a value below for non-String columns)<br><br>");
            
            result.append("INSERT INTO ");
            
            result.append("<input name=\"table\" size=\"16\" value=\"tableName\"> ");
            result.append("(<input name=\"columns\" size=\"40\" value=\"column1, column2, ...\">) ");
            // Sorry, no commata allowed in string values for the moment...
            result.append("VALUES <input name=\"data\" size=\"40\" value=\"value1, value2, ...\">;<br>");
            
            result.append("<input type=\"submit\">");
            result.append("</form>");
        }
        
        result.append(getHtmlTail());
        
        return result.toString();
    }
}
